@@ -5,20 +5,16 @@
 */
 
 #include "plasmadesktoptheme.h"
-#include <KIconLoader>
-#include <QDebug>
+
+#include <QDBusConnection>
 #include <QGuiApplication>
 #include <QPalette>
-#include <QQmlContext>
-#include <QQmlEngine>
 #include <QQuickRenderControl>
 #include <QQuickWindow>
-#include <QScopeGuard>
 
 #include <KColorScheme>
 #include <KConfigGroup>
 #include <KIconColors>
-#include <QDBusConnection>
 
 class StyleSingleton : public QObject
 {
@@ -36,8 +32,6 @@ public:
         , buttonScheme(QPalette::Active, KColorScheme::ColorSet::Button)
         , viewScheme(QPalette::Active, KColorScheme::ColorSet::View)
     {
-        connect(qGuiApp, &QGuiApplication::paletteChanged, this, &StyleSingleton::refresh);
-
         // Use DBus in order to listen for settings changes directly, as the
         // QApplication doesn't expose the font variants we're looking for,
         // namely smallFont.
@@ -49,7 +43,7 @@ public:
                                               SLOT(notifyWatchersConfigurationChange()));
 
         connect(qGuiApp, &QGuiApplication::fontDatabaseChanged, this, &StyleSingleton::notifyWatchersConfigurationChange);
-        connect(qGuiApp, &QGuiApplication::fontChanged, this, &StyleSingleton::notifyWatchersConfigurationChange);
+        qGuiApp->installEventFilter(this);
 
         // Use NativeTextRendering as the default text rendering type when the scale factor is an integer.
         // NativeTextRendering is still distorted sometimes with fractional scale factors,
@@ -65,7 +59,7 @@ public:
     QFont loadSmallFont() const
     {
         KSharedConfigPtr ptr = KSharedConfig::openConfig();
-        KConfigGroup general(ptr->group("general"));
+        KConfigGroup general(ptr->group(QStringLiteral("general")));
 
         return general.readEntry("smallestReadableFont", []() {
             auto smallFont = qApp->font();
@@ -89,7 +83,7 @@ public:
         notifyWatchersPaletteChange();
     }
 
-    Colors loadColors(Kirigami::PlatformTheme::ColorSet cs, QPalette::ColorGroup group)
+    Colors loadColors(Kirigami::Platform::PlatformTheme::ColorSet cs, QPalette::ColorGroup group)
     {
         const auto key = qMakePair(cs, group);
         auto it = m_cache.constFind(key);
@@ -97,7 +91,7 @@ public:
             return *it;
         }
 
-        using Kirigami::PlatformTheme;
+        using Kirigami::Platform::PlatformTheme;
 
         KColorScheme::ColorSet set;
 
@@ -175,10 +169,22 @@ public:
     KColorScheme viewScheme;
     QFont smallFont;
 
-    QVector<PlasmaDesktopTheme *> watchers;
+    QList<PlasmaDesktopTheme *> watchers;
+
+protected:
+    bool eventFilter([[maybe_unused]] QObject *obj, QEvent *event) override
+    {
+        if (event->type() == QEvent::ApplicationFontChange) {
+            notifyWatchersConfigurationChange();
+        }
+        if (event->type() == QEvent::PaletteChange) {
+            refresh();
+        }
+        return false;
+    }
 
 private:
-    QHash<QPair<Kirigami::PlatformTheme::ColorSet, QPalette::ColorGroup>, Colors> m_cache;
+    QHash<QPair<Kirigami::Platform::PlatformTheme::ColorSet, QPalette::ColorGroup>, Colors> m_cache;
 };
 
 Q_GLOBAL_STATIC(StyleSingleton, s_style);
@@ -304,15 +310,15 @@ void PlasmaDesktopTheme::syncColors()
 
 bool PlasmaDesktopTheme::event(QEvent *event)
 {
-    if (event->type() == Kirigami::PlatformThemeEvents::DataChangedEvent::type) {
+    if (event->type() == Kirigami::Platform::PlatformThemeEvents::DataChangedEvent::type) {
         syncColors();
     }
 
-    if (event->type() == Kirigami::PlatformThemeEvents::ColorSetChangedEvent::type) {
+    if (event->type() == Kirigami::Platform::PlatformThemeEvents::ColorSetChangedEvent::type) {
         syncColors();
     }
 
-    if (event->type() == Kirigami::PlatformThemeEvents::ColorGroupChangedEvent::type) {
+    if (event->type() == Kirigami::Platform::PlatformThemeEvents::ColorGroupChangedEvent::type) {
         syncColors();
     }
 
