@@ -17,14 +17,14 @@
 #include <QStringBuilder>
 #include <QStyle>
 #include <QStyleFactory>
-#include <QtQuick/qsgninepatchnode.h>
+#include <qsgninepatchnode.h>
 #include <qstyleoption.h>
 
 #include <KConfigGroup>
 #include <ksharedconfig.h>
 
-#include <Kirigami/PlatformTheme>
-#include <Kirigami/TabletModeWatcher>
+#include <Kirigami/Platform/PlatformTheme>
+#include <Kirigami/Platform/TabletModeWatcher>
 
 std::unique_ptr<QStyle> KQuickStyleItem::s_style = nullptr;
 
@@ -37,6 +37,11 @@ QStyle *KQuickStyleItem::style()
     }
 }
 
+static bool isKeyFocusReason(Qt::FocusReason reason)
+{
+    return reason == Qt::TabFocusReason || reason == Qt::BacktabFocusReason || reason == Qt::ShortcutFocusReason;
+}
+
 KQuickStyleItem::KQuickStyleItem(QQuickItem *parent)
     : QQuickItem(parent)
     , m_styleoption(nullptr)
@@ -61,7 +66,7 @@ KQuickStyleItem::KQuickStyleItem(QQuickItem *parent)
     , m_contentHeight(0)
     , m_textureWidth(0)
     , m_textureHeight(0)
-    , m_lastFocusReason(Qt::NoFocusReason)
+    , m_focusReason(Qt::NoFocusReason)
 {
     // Check that we really are a QApplication before attempting to access the
     // application style.
@@ -95,7 +100,7 @@ KQuickStyleItem::KQuickStyleItem(QQuickItem *parent)
         // We are not a QApplication.  Create an internal copy of the configured
         // desktop style, to be used for metrics, options and painting.
         KSharedConfig::Ptr kdeglobals = KSharedConfig::openConfig();
-        KConfigGroup cg(kdeglobals, "KDE");
+        KConfigGroup cg(kdeglobals, QStringLiteral("KDE"));
         s_style.reset(QStyleFactory::create(cg.readEntry("widgetStyle", QStringLiteral("Fusion"))));
     }
 
@@ -103,9 +108,9 @@ KQuickStyleItem::KQuickStyleItem(QQuickItem *parent)
     setFlag(QQuickItem::ItemHasContents, true);
     setSmooth(false);
 
-    connect(qApp, &QApplication::fontChanged, this, &KQuickStyleItem::updateSizeHint, Qt::QueuedConnection);
+    qGuiApp->installEventFilter(this);
 
-    Kirigami::TabletModeWatcher::self()->addWatcher(this);
+    Kirigami::Platform::TabletModeWatcher::self()->addWatcher(this);
 }
 
 KQuickStyleItem::~KQuickStyleItem()
@@ -145,19 +150,19 @@ KQuickStyleItem::~KQuickStyleItem()
     }
 
     m_styleoption = nullptr;
-    Kirigami::TabletModeWatcher::self()->removeWatcher(this);
+    Kirigami::Platform::TabletModeWatcher::self()->removeWatcher(this);
 }
 
 void KQuickStyleItem::initStyleOption()
 {
     if (!m_theme) {
-        m_theme = static_cast<Kirigami::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::PlatformTheme>(this, true));
+        m_theme = static_cast<Kirigami::Platform::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::Platform::PlatformTheme>(this, true));
         Q_ASSERT(m_theme);
 
-        connect(m_theme, &Kirigami::PlatformTheme::colorsChanged, this, [this]() {
+        connect(m_theme, &Kirigami::Platform::PlatformTheme::colorsChanged, this, [this]() {
             // we need to reset the palette event if Qt::AA_SetPalette attribute has been set
             m_styleoption->palette = m_theme->palette();
-            updateItem();
+            polish();
         });
     }
     Q_ASSERT(m_theme);
@@ -288,7 +293,10 @@ void KQuickStyleItem::initStyleOption()
         } else {
             opt->position = QStyleOptionHeader::Middle;
         }
-
+        opt->orientation = m_properties.value(QStringLiteral("orientation")).value<Qt::Orientation>();
+        if (opt->orientation == Qt::Orientation(0)) {
+            opt->orientation = Qt::Horizontal;
+        }
         const QFont font = qApp->font("QHeaderView");
         opt->fontMetrics = QFontMetrics(font);
         break;
@@ -504,11 +512,7 @@ void KQuickStyleItem::initStyleOption()
                 QString shortcut = m_properties[QStringLiteral("shortcut")].toString();
                 if (!shortcut.isEmpty()) {
                     opt->text += QLatin1Char('\t') + shortcut;
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-                    opt->tabWidth = qMax(opt->tabWidth, qRound(textWidth(shortcut)));
-#else
                     opt->reservedShortcutWidth = qMax(opt->reservedShortcutWidth, qRound(textWidth(shortcut)));
-#endif
                 }
 
                 if (m_properties[QStringLiteral("checkable")].toBool()) {
@@ -799,7 +803,7 @@ void KQuickStyleItem::initStyleOption()
     // some styles don't draw a focus rectangle if
     // QStyle::State_KeyboardFocusChange is not set
     if (window()) {
-        if (m_lastFocusReason == Qt::TabFocusReason || m_lastFocusReason == Qt::BacktabFocusReason) {
+        if (isKeyFocusReason(m_focusReason)) {
             m_styleoption->state |= QStyle::State_KeyboardFocusChange;
         }
     }
@@ -816,12 +820,12 @@ QIcon KQuickStyleItem::iconFromIconProperty() const
 {
     QIcon icon;
     const QVariant iconProperty = m_properties[QStringLiteral("icon")];
-    switch (iconProperty.type()) {
-    case QVariant::Icon:
+    switch (iconProperty.userType()) {
+    case QMetaType::QIcon:
         icon = iconProperty.value<QIcon>();
         break;
-    case QVariant::Url:
-    case QVariant::String: {
+    case QMetaType::QUrl:
+    case QMetaType::QString: {
         QString iconSource = iconProperty.toString();
         if (iconSource.startsWith(QLatin1String("qrc:/"))) {
             iconSource = iconSource.mid(3);
@@ -898,52 +902,42 @@ void KQuickStyleItem::resolvePalette()
     }
 }
 
-int KQuickStyleItem::leftPadding() const
-{
-    switch (m_itemType) {
-    case Frame: {
-        const QRect cr = KQuickStyleItem::style()->subElementRect(QStyle::SE_ShapedFrameContents, m_styleoption);
-        return cr.left() - m_styleoption->rect.left();
-    }
-    default:
-        return 0;
-    }
-}
-
 int KQuickStyleItem::topPadding() const
 {
-    switch (m_itemType) {
-    case Frame: {
-        const QRect cr = KQuickStyleItem::style()->subElementRect(QStyle::SE_ShapedFrameContents, m_styleoption);
-        return cr.top() - m_styleoption->rect.top();
-    }
-    default:
-        return 0;
-    }
+    return padding(Qt::TopEdge);
+}
+
+int KQuickStyleItem::leftPadding() const
+{
+    return padding(Qt::LeftEdge);
 }
 
 int KQuickStyleItem::rightPadding() const
 {
-    switch (m_itemType) {
-    case Frame: {
-        const QRect cr = KQuickStyleItem::style()->subElementRect(QStyle::SE_ShapedFrameContents, m_styleoption);
-        return m_styleoption->rect.right() - cr.right();
-    }
-    default:
-        return 0;
-    }
+    return padding(Qt::RightEdge);
 }
 
 int KQuickStyleItem::bottomPadding() const
 {
-    switch (m_itemType) {
-    case Frame: {
+    return padding(Qt::BottomEdge);
+}
+
+int KQuickStyleItem::padding(Qt::Edge edge) const
+{
+    if (m_itemType == Frame) {
         const QRect cr = KQuickStyleItem::style()->subElementRect(QStyle::SE_ShapedFrameContents, m_styleoption);
-        return m_styleoption->rect.bottom() - cr.bottom();
-    }
-    default:
-        return 0;
+        switch (edge) {
+        case Qt::TopEdge:
+            return cr.top() - m_styleoption->rect.top();
+        case Qt::LeftEdge:
+            return cr.left() - m_styleoption->rect.left();
+        case Qt::RightEdge:
+            return m_styleoption->rect.right() - cr.right();
+        case Qt::BottomEdge:
+            return m_styleoption->rect.bottom() - cr.bottom();
+        }
     }
+    return 0;
 }
 
 /*
@@ -958,15 +952,16 @@ int KQuickStyleItem::bottomPadding() const
 
 QString KQuickStyleItem::styleName() const
 {
-    QString style = QString::fromLatin1(KQuickStyleItem::style()->metaObject()->className());
-    style = style.toLower();
-    if (style.startsWith(QLatin1Char('q'))) {
-        style = style.right(style.length() - 1);
+    const QString fullName = QString::fromLatin1(KQuickStyleItem::style()->metaObject()->className());
+    QStringView shortName = fullName;
+    if (shortName.startsWith(QLatin1Char('q'), Qt::CaseInsensitive)) {
+        shortName = shortName.sliced(1);
     }
-    if (style.endsWith(QLatin1String("style"))) {
-        style = style.left(style.length() - 5);
+    const QLatin1String suffix("style");
+    if (shortName.endsWith(suffix, Qt::CaseInsensitive)) {
+        shortName.chop(suffix.length());
     }
-    return style;
+    return shortName.toString().toLower();
 }
 
 QString KQuickStyleItem::hitTest(int px, int py)
@@ -1142,12 +1137,46 @@ QSize KQuickStyleItem::sizeFromContents(int width, int height)
 
         break;
     }
-    case Slider:
-        size = KQuickStyleItem::style()->sizeFromContents(QStyle::CT_Slider, m_styleoption, QSize(width, height));
+    case Slider: {
+        const auto opt = qstyleoption_cast<const QStyleOptionSlider *>(m_styleoption);
+        const auto tickPosition = opt->tickPosition;
+
+        // Copied from QSlider
+        const int SliderLength = 84, TickSpace = 5;
+        int thick = style()->pixelMetric(QStyle::PM_SliderThickness, m_styleoption, nullptr);
+        if (tickPosition & QSlider::TicksAbove) {
+            thick += TickSpace;
+        }
+        if (tickPosition & QSlider::TicksBelow) {
+            thick += TickSpace;
+        }
+        int w = thick;
+        int h = SliderLength;
+        if (opt->orientation == Qt::Horizontal) {
+            w = SliderLength;
+            h = thick;
+        }
+        size = KQuickStyleItem::style()->sizeFromContents(QStyle::CT_Slider, m_styleoption, QSize(w, h));
         break;
+    }
     case ProgressBar:
         size = KQuickStyleItem::style()->sizeFromContents(QStyle::CT_ProgressBar, m_styleoption, QSize(width, height));
         break;
+
+    case ScrollBar: {
+        const auto opt = qstyleoption_cast<const QStyleOptionSlider *>(m_styleoption);
+
+        // Copied from QScrollBar
+        const int scrollBarExtent = KQuickStyleItem::style()->pixelMetric(QStyle::PM_ScrollBarExtent, opt, nullptr);
+        const int scrollBarSliderMin = KQuickStyleItem::style()->pixelMetric(QStyle::PM_ScrollBarSliderMin, opt, nullptr);
+        if (opt->orientation == Qt::Horizontal) {
+            size = QSize(scrollBarExtent * 2 + scrollBarSliderMin, scrollBarExtent);
+        } else {
+            size = QSize(scrollBarExtent, scrollBarExtent * 2 + scrollBarSliderMin);
+        }
+        size = KQuickStyleItem::style()->sizeFromContents(QStyle::CT_ScrollBar, opt, size);
+        break;
+    }
     case SpinBox:
     case Edit: {
         // We have to create a new style option since we might be calling with a QStyleOptionSpinBox
@@ -1397,10 +1426,10 @@ QVariant KQuickStyleItem::styleHint(const QString &metric)
     // Add SH_Menu_SpaceActivatesItem
 }
 
-void KQuickStyleItem::setHints(const QVariantMap &str)
+void KQuickStyleItem::setHints(const QVariantMap &hints)
 {
-    if (m_hints != str) {
-        m_hints = str;
+    if (m_hints != hints) {
+        m_hints = hints;
         initStyleOption();
         updateSizeHint();
         polish();
@@ -1572,32 +1601,6 @@ QRectF KQuickStyleItem::subControlRect(const QString &subcontrolString)
     return QRectF();
 }
 
-namespace
-{
-class QHighDpiPixmapsEnabler1
-{
-public:
-    QHighDpiPixmapsEnabler1()
-        : wasEnabled(false)
-    {
-        if (!qApp->testAttribute(Qt::AA_UseHighDpiPixmaps)) {
-            qApp->setAttribute(Qt::AA_UseHighDpiPixmaps);
-            wasEnabled = true;
-        }
-    }
-
-    ~QHighDpiPixmapsEnabler1()
-    {
-        if (wasEnabled) {
-            qApp->setAttribute(Qt::AA_UseHighDpiPixmaps, false);
-        }
-    }
-
-private:
-    bool wasEnabled;
-};
-}
-
 void KQuickStyleItem::paint(QPainter *painter)
 {
     initStyleOption();
@@ -1615,11 +1618,6 @@ void KQuickStyleItem::paint(QPainter *painter)
         painter->setFont(font);
     }
 
-    // Set AA_UseHighDpiPixmaps when calling style code to make QIcon return
-    // "retina" pixmaps. The flag is controlled by the application so we can't
-    // set it unconditionally.
-    QHighDpiPixmapsEnabler1 enabler;
-
     switch (m_itemType) {
     case Button:
         KQuickStyleItem::style()->drawControl(QStyle::CE_PushButton, m_styleoption, painter);
@@ -1657,7 +1655,7 @@ void KQuickStyleItem::paint(QPainter *painter)
         KQuickStyleItem::style()->drawComplexControl(QStyle::CC_ToolButton, qstyleoption_cast<QStyleOptionComplex *>(m_styleoption), painter);
         break;
     case Tab: {
-        if (m_lastFocusReason != Qt::TabFocusReason && m_lastFocusReason != Qt::BacktabFocusReason) {
+        if (!isKeyFocusReason(m_focusReason)) {
             m_styleoption->state &= ~QStyle::State_HasFocus;
         }
         KQuickStyleItem::style()->drawControl(QStyle::CE_TabBarTab, m_styleoption, painter);
@@ -1839,6 +1837,17 @@ bool KQuickStyleItem::hasThemeIcon(const QString &icon) const
     return QIcon::hasThemeIcon(icon);
 }
 
+void KQuickStyleItem::updateFocusReason()
+{
+    if (m_control) {
+        auto reason = m_focusReasonProperty.read(m_control).value<Qt::FocusReason>();
+        if (m_focusReason != reason) {
+            m_focusReason = reason;
+            polish();
+        }
+    }
+}
+
 bool KQuickStyleItem::event(QEvent *ev)
 {
     if (ev->type() == QEvent::StyleAnimationUpdate) {
@@ -1847,7 +1856,7 @@ bool KQuickStyleItem::event(QEvent *ev)
             polish();
         }
         return true;
-    } else if (ev->type() == Kirigami::TabletModeChangedEvent::type) {
+    } else if (ev->type() == Kirigami::Platform::TabletModeChangedEvent::type) {
         Q_EMIT leftPaddingChanged();
         Q_EMIT rightPaddingChanged();
         Q_EMIT topPaddingChanged();
@@ -1897,8 +1906,21 @@ void KQuickStyleItem::setControl(QQuickItem *control)
     }
 
     m_control = control;
+    m_focusReasonProperty = {};
+    m_focusReason = Qt::NoFocusReason;
 
     if (m_control) {
+        // focusReasonChanged is a signal of QQuickControl class which is not exposed.
+        const auto mo = m_control->metaObject();
+        m_focusReasonProperty = mo->property(mo->indexOfProperty("focusReason"));
+        if (m_focusReasonProperty.isValid()) {
+            // wish there was something like QMetaMethod::fromSignal but for slots
+            const auto slot = metaObject()->method(metaObject()->indexOfSlot("updateFocusReason()"));
+            // also wish there was a way to connect QMetaMethod to arbitrary member function
+            connect(m_control, m_focusReasonProperty.notifySignal(), this, slot);
+            updateFocusReason();
+        }
+
         m_control->installEventFilter(this);
 
         if (m_control->window()) {
@@ -1969,10 +1991,6 @@ void KQuickStyleItem::updatePolish()
 bool KQuickStyleItem::eventFilter(QObject *watched, QEvent *event)
 {
     if (watched == m_control) {
-        if (event->type() == QEvent::FocusIn || event->type() == QEvent::FocusOut) {
-            QFocusEvent *fe = static_cast<QFocusEvent *>(event);
-            m_lastFocusReason = fe->reason();
-        }
         // Page accepts mouse events without doing anything with them (for a workaround wrt dragging from empty areas of flickables) when the interaction is
         // pure mouse, steal events from them, so a parent handler can initiate a window drag from empty areas, either Kirigami.ApplicationWindow or the breeze
         // style from a QQwuickwidget
@@ -1987,9 +2005,13 @@ bool KQuickStyleItem::eventFilter(QObject *watched, QEvent *event)
         if (event->type() == QEvent::KeyPress || event->type() == QEvent::KeyRelease) {
             QKeyEvent *ke = static_cast<QKeyEvent *>(event);
             if (ke->key() == Qt::Key_Alt) {
-                updateItem();
+                polish();
             }
         }
+    } else if (watched == qGuiApp) {
+        if (event->type() == QEvent::ApplicationFontChange) {
+            QMetaObject::invokeMethod(this, &KQuickStyleItem::updateSizeHint, Qt::QueuedConnection);
+        }
     }
 
     return QQuickItem::eventFilter(watched, event);
@@ -2004,15 +2026,9 @@ void KQuickStyleItem::itemChange(QQuickItem::ItemChange change, const QQuickItem
     QQuickItem::itemChange(change, value);
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-void KQuickStyleItem::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
-{
-    QQuickItem::geometryChanged(newGeometry, oldGeometry);
-#else
 void KQuickStyleItem::geometryChange(const QRectF &newGeometry, const QRectF &oldGeometry)
 {
     QQuickItem::geometryChange(newGeometry, oldGeometry);
-#endif
 
     if (!newGeometry.isEmpty() && newGeometry != oldGeometry) {
         polish();
@@ -2039,7 +2055,7 @@ void KQuickStyleItem::styleChanged()
     connect(style, &QObject::destroyed, this, &KQuickStyleItem::styleChanged);
 
     updateSizeHint();
-    updateItem();
+    polish();
     Q_EMIT styleNameChanged();
 }
 
